==> dcoupon_sdk_flutter_package-1.1.3.tar.gz: lib/src/data/datasources/user_remote_data_source.dart <==
import 'dart:convert';
import 'dart:io';

import 'package:dartz/dartz.dart';
import 'package:dcoupon_sdk_flutter_package/src/config_reader.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/exceptions.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/helpers/third_party_api_helper.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/helpers/users_api_helper.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/legal_document_model.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_accept_enrollment_form_request.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_accept_enrollment_form_response.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_add_or_update_loyalty_card_request.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_delete_loyalty_card_request.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_get_enrollment_form_response.dart';

import 'package:dcoupon_sdk_flutter_package/src/data/models/user_login_request.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_login_response.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_update_user_data_request.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/entities/user.dart';
import 'package:dcoupon_sdk_flutter_package/src/utils/constants.dart';
import 'package:dio/dio.dart';
import 'package:meta/meta.dart';

abstract class UserRemoteDataSource {
  Future<Either<Failure, UserLoginResponse>> logIn({
    @required String externalId,
    @required User user,
  });

  Future<Either<Failure, UserGetEnrollmentFormResponse>> getEnrollmentForm();

  Future<Either<Failure, UserAcceptEnrollmentFormResponse>>
      acceptEnrollmentForm({
    @required String externalId,
    @required User user,
    @required List<LegalDocumentModel> legalDocumentsAccepted,
  });

  Future<Either<Failure, dynamic>> getUserData();

  Future<Either<Failure, dynamic>> updateUserData({
    String alias,
    String gender,
    String birthdate,
  });

  Future<Either<Failure, dynamic>> getLoyaltyAffiliates();

  Future<Either<Failure, dynamic>> getUserLoyaltyCards();

  Future<Either<Failure, dynamic>> addOrUpdateLoyaltyCard({
    String affiliateApiToken,
    String cardNumber,
  });

  Future<Either<Failure, dynamic>> deleteLoyaltyCard({
    String affiliateApiToken,
    String cardNumber,
  });
}

class UserRemoteDataSourceImpl implements UserRemoteDataSource {
  Dio dio = Dio();

  @override
  Future<Either<Failure, UserLoginResponse>> logIn({
    @required String externalId,
    @required User user,
  }) async {
    dio.options.contentType = Headers.jsonContentType;

    dio.options.headers = ThirdPartyAPIHelper.buildHeaders();

    final userLoginRequest = UserLoginRequest(
      clientCountry: ConfigReader.clientCountry,
      externalId: externalId,
      user: user,
    );

    try {
      final response = await dio.post(
        ThirdPartyAPIHelper.userLoginUrl(),
        data: userLoginRequest.toJson(),
      );
      final userLoginResponse = UserLoginResponse.asCompleted(response);
      return Right(userLoginResponse);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        if (e.response.statusCode == HttpStatus.movedPermanently) {
          final headersMap = e.response.headers.map;
          final enrollmentUrl = headersMap[Constants.HEADER_LOCATION].first;
          final enrollmentUri = Uri.parse(enrollmentUrl);
          final jwt = enrollmentUri.queryParameters[Constants.JWT];
          return Right(UserLoginResponse.asEnrollmentNeeded(jwt));
        }
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, UserGetEnrollmentFormResponse>>
      getEnrollmentForm() async {
    dio.options.headers =
        ThirdPartyAPIHelper.buildHeaders(authMethod: AuthMethod.JWT);

    try {
      final response = await dio.get(
        ThirdPartyAPIHelper.getEnrollmentFormUrl(),
      );
      final userEnrollmentFormResponse =
          UserGetEnrollmentFormResponse.fromJson(response.data);
      return Right(userEnrollmentFormResponse);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, UserAcceptEnrollmentFormResponse>>
      acceptEnrollmentForm({
    String externalId,
    @required User user,
    @required List<LegalDocumentModel> legalDocumentsAccepted,
  }) async {
    dio.options.contentType = Headers.jsonContentType;

    dio.options.headers = ThirdPartyAPIHelper.buildHeaders();

    final userAcceptEnrollmentFormRequest = UserAcceptEnrollmentFormRequest(
      jwt: ConfigReader.userJwt,
      externalId: externalId,
      user: user,
      legalDocumentsAccepted: legalDocumentsAccepted,
    );

    try {
      final data = jsonEncode(userAcceptEnrollmentFormRequest.toJson());
      final response = await dio.post(
        ThirdPartyAPIHelper.acceptEnrollmentFormUrl(),
        data: data,
      );
      final userAcceptEnrollmentFormResponse =
          UserAcceptEnrollmentFormResponse.fromResponse(response);
      return Right(userAcceptEnrollmentFormResponse);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getUserData() async {
    dio.options.headers = UsersApiHelper.buildHeaders();

    try {
      final response = await dio.get(
        UsersApiHelper.getUserDataUrl(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> updateUserData({
    String alias,
    String gender,
    String birthdate,
  }) async {
    dio.options.contentType = Headers.jsonContentType;

    dio.options.headers = UsersApiHelper.buildHeaders();

    final userUpdateUserDataRequest = UserUpdateUserDataRequest(
      alias: alias,
      gender: gender,
      birthdate: birthdate,
    );

    try {
      final response = await dio.post(
        UsersApiHelper.updateUserDataUrl(),
        data: userUpdateUserDataRequest.toJson(),
      );

      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getLoyaltyAffiliates() async {
    dio.options.headers = UsersApiHelper.buildHeaders(
      userAuthentication: false,
    );

    try {
      final response = await dio.get(
        UsersApiHelper.getLoyaltyAffiliatesUrl(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getUserLoyaltyCards() async {
    dio.options.headers = UsersApiHelper.buildHeaders();

    try {
      final response = await dio.get(
        UsersApiHelper.getUserLoyaltyCardsUrl(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> addOrUpdateLoyaltyCard({
    String affiliateApiToken,
    String cardNumber,
  }) async {
    dio.options.contentType = Headers.jsonContentType;

    dio.options.headers = UsersApiHelper.buildHeaders();

    final userAddOrUpdateLoyaltyCardRequest = UserAddOrUpdateLoyaltyCardRequest(
      affiliateApiToken: affiliateApiToken,
      cardNumber: cardNumber,
    );

    try {
      final response = await dio.post(
        UsersApiHelper.addOrUpdateLoyaltyCardUrl(),
        data: userAddOrUpdateLoyaltyCardRequest.toJson(),
      );

      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> deleteLoyaltyCard({
    String affiliateApiToken,
    String cardNumber,
  }) async {
    dio.options.contentType = Headers.jsonContentType;

    dio.options.headers = UsersApiHelper.buildHeaders();

    final userDeleteLoyaltyCardRequest = UserDeleteLoyaltyCardRequest(
      affiliateApiToken: affiliateApiToken,
      cardNumber: cardNumber,
    );

    try {
      final response = await dio.delete(
        UsersApiHelper.deleteLoyaltyCardUrl(),
        data: userDeleteLoyaltyCardRequest.toJson(),
      );

      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }
}

==> dcoupon_sdk_flutter_package-2.0.0.tar.gz: lib/src/data/datasources/user_remote_data_source.dart <==
import 'dart:convert';
import 'dart:io';

import 'package:dartz/dartz.dart';
import 'package:dcoupon_sdk_flutter_package/src/config_reader.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/exceptions.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/dio_logging_interceptor.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/helpers/third_party_api_helper.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/helpers/users_api_helper.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/legal_document_model.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_accept_enrollment_form_request.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_accept_enrollment_form_response.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_add_or_update_loyalty_card_request.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_delete_loyalty_card_request.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_get_enrollment_form_response.dart';

import 'package:dcoupon_sdk_flutter_package/src/data/models/user_login_request.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_login_response.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_update_user_data_request.dart';
import 'package:dcoupon_sdk_flutter_package/src/utils/constants.dart';
import 'package:dio/dio.dart';
import 'package:meta/meta.dart';

abstract class UserRemoteDataSource {
  Future<Either<Failure, UserLoginResponse>> login({
    @required String externalId,
    @required String email,
    String referralCode,
  });

  Future<Either<Failure, UserGetEnrollmentFormResponse>> getEnrollmentForm({
    @required String jwt,
  });

  Future<Either<Failure, UserAcceptEnrollmentFormResponse>>
      acceptEnrollmentForm({
    @required String jwt,
    @required List<LegalDocumentModel> legalDocumentsAccepted,
  });

  Future<Either<Failure, dynamic>> getUserData();

  Future<Either<Failure, dynamic>> updateUserData({
    String alias,
    String gender,
    String birthdate,
  });

  Future<Either<Failure, dynamic>> getLoyaltyAffiliates();

  Future<Either<Failure, dynamic>> getUserLoyaltyCards();

  Future<Either<Failure, dynamic>> addOrUpdateLoyaltyCard({
    String affiliateApiToken,
    String cardNumber,
  });

  Future<Either<Failure, dynamic>> deleteLoyaltyCard({
    String affiliateApiToken,
    String cardNumber,
  });
}

class UserRemoteDataSourceImpl implements UserRemoteDataSource {
  Dio dio = Dio();

  @override
  Future<Either<Failure, UserLoginResponse>> login({
    @required String externalId,
    @required String email,
    String referralCode,
  }) async {
    dio.options.contentType = Headers.jsonContentType;

    dio.options.headers = ThirdPartyAPIHelper.buildHeaders();

    final userLoginRequest = UserLoginRequest(
      clientCountry: ConfigReader.clientCountry,
      externalId: externalId,
      email: email,
      referralCode: referralCode,
    );

    try {
      final response = await dio.post(
        ThirdPartyAPIHelper.userLoginUrl(),
        data: userLoginRequest.toJson(),
      );
      final userLoginResponse = UserLoginResponse.asCompleted(response);
      return Right(userLoginResponse);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        if (e.response.statusCode == HttpStatus.movedPermanently) {
          final headersMap = e.response.headers.map;
          final enrollmentUrl = headersMap[Constants.HEADER_LOCATION].first;
          final enrollmentUri = Uri.parse(enrollmentUrl);
          final jwt = enrollmentUri.queryParameters[Constants.JWT];
          return Right(UserLoginResponse.asEnrollmentNeeded(jwt));
        }
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, UserGetEnrollmentFormResponse>> getEnrollmentForm({
    @required String jwt,
  }) async {
    dio.options.headers = ThirdPartyAPIHelper.buildHeaders(
      authMethod: AuthMethod.JWT,
      jwt: jwt,
    );

    try {
      final response = await dio.get(
        ThirdPartyAPIHelper.getEnrollmentFormUrl(),
      );
      final userEnrollmentFormResponse =
          UserGetEnrollmentFormResponse.fromJson(response.data);
      return Right(userEnrollmentFormResponse);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, UserAcceptEnrollmentFormResponse>>
      acceptEnrollmentForm({
    @required String jwt,
    @required List<LegalDocumentModel> legalDocumentsAccepted,
  }) async {
    dio.options.contentType = Headers.jsonContentType;

    dio.options.headers = ThirdPartyAPIHelper.buildHeaders();

    final userAcceptEnrollmentFormRequest = UserAcceptEnrollmentFormRequest(
      jwt: jwt,
      legalDocumentsAccepted: legalDocumentsAccepted,
    );

    try {
      final data = jsonEncode(userAcceptEnrollmentFormRequest.toJson());
      final response = await dio.post(
        ThirdPartyAPIHelper.acceptEnrollmentFormUrl(),
        data: data,
      );
      final userAcceptEnrollmentFormResponse =
          UserAcceptEnrollmentFormResponse.fromResponse(response);
      return Right(userAcceptEnrollmentFormResponse);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getUserData() async {
    dio.options.headers = UsersApiHelper.buildHeaders();

    try {
      final response = await dio.get(
        UsersApiHelper.getUserDataUrl(),
      );

      final data = response.data as Map<String, dynamic>;

      return Right(data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> updateUserData({
    String alias,
    String gender,
    String birthdate,
  }) async {
    dio.options.contentType = Headers.jsonContentType;

    dio.options.headers = UsersApiHelper.buildHeaders();

    final userUpdateUserDataRequest = UserUpdateUserDataRequest(
      alias: alias,
      gender: gender,
      birthdate: birthdate,
    );

    try {
      final response = await dio.post(
        UsersApiHelper.updateUserDataUrl(),
        data: userUpdateUserDataRequest.toJson(),
      );

      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getLoyaltyAffiliates() async {
    dio.options.headers = UsersApiHelper.buildHeaders(
      userAuthentication: false,
    );

    try {
      final response = await dio.get(
        UsersApiHelper.getLoyaltyAffiliatesUrl(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getUserLoyaltyCards() async {
    dio.options.headers = UsersApiHelper.buildHeaders();

    try {
      final response = await dio.get(
        UsersApiHelper.getUserLoyaltyCardsUrl(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> addOrUpdateLoyaltyCard({
    String affiliateApiToken,
    String cardNumber,
  }) async {
    dio.options.contentType = Headers.jsonContentType;

    dio.options.headers = UsersApiHelper.buildHeaders();

    final userAddOrUpdateLoyaltyCardRequest = UserAddOrUpdateLoyaltyCardRequest(
      affiliateApiToken: affiliateApiToken,
      cardNumber: cardNumber,
    );

    try {
      final response = await dio.post(
        UsersApiHelper.addOrUpdateLoyaltyCardUrl(),
        data: userAddOrUpdateLoyaltyCardRequest.toJson(),
      );

      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> deleteLoyaltyCard({
    String affiliateApiToken,
    String cardNumber,
  }) async {
    dio.options.contentType = Headers.jsonContentType;

    dio.options.headers = UsersApiHelper.buildHeaders();

    final userDeleteLoyaltyCardRequest = UserDeleteLoyaltyCardRequest(
      affiliateApiToken: affiliateApiToken,
      cardNumber: cardNumber,
    );

    try {
      final response = await dio.delete(
        UsersApiHelper.deleteLoyaltyCardUrl(),
        data: userDeleteLoyaltyCardRequest.toJson(),
      );

      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }
}
